unit uAmxxModule;

interface

uses
  Classes, SysUtils, StrUtils, Windows, WinSock, Math, uMetaAPI, php4delphi,
  PHPApi, zendAPI, IniFiles;

{$INCLUDE moduleconfig.inc}

// ***** AMXX stuff *****

// module interface version was 1  ģӿڰ汾Ϊ 1
// 2 - added logtag to struct (amxx1.1-rc1)  logtag ṹ
// 3 - added new tagAMX structure (amxx1.5) µ tagAMX ṹ
// 4 - added new 'library' setting for direct loading µ 'library' á
const AMXX_INTERFACE_VERSION = 4;

const
  AMXX_OK               = 0;     // no error
  AMXX_IFVERS           = 1;     // interface version
  AMXX_PARAM            = 2;     // Invalid parameter
  AMXX_FUNC_NOT_PRESENT = 3;     // Function not present

type
  PAmxx_Module_Info_s = ^TAmxx_Module_Info_s;
  TAmxx_Module_Info_s = packed record
    Name: PChar;
    Author: PChar;
    Version: PChar;
    Reload: Integer;  // reload on mapchange when nonzero
    LogTag: PChar;    // added in version 2
    _Library: PChar;  // added in version 4
    LibClass: PChar;  // added in version 4
  end;

type PFN_REQ_FNPTR = function(name: PChar): Pointer; cdecl;

type
  Cell = type Integer;
  PCell = ^Cell;
  PPCell = ^PCell;

const AMX_USERNUM = 4;

type
  PtagAMX = ^tagAMX;
  PAMX = PtagAMX;

  AMX_NATIVE = function(amx: PAMX; Params: PCell): Cell; cdecl;
  AMX_CALLBACK = function(amx:PAMX; Index: Cell; result, params: PCell): Integer; cdecl;
  AMX_DEBUG = function(amx:PAMX): Integer; cdecl;

  tagAMX = packed record
    base: PByte;            // points to the AMX header plus the code, optionally also the data
    data: PByte;            // points to separate data+stack+heap, may be NULL
    callback: AMX_CALLBACK;
    debug: AMX_DEBUG;       // debug callback
    // for external functions a few registers must be accessible from the outside
    cip: Cell;              // instruction pointer: relative to base + amxhdr->cod
    frm: Cell;              // stack frame base: relative to base + amxhdr->dat
    hea: Cell;              // top of the heap: relative to base + amxhdr->dat
    hlw: Cell;              // bottom of the heap: relative to base + amxhdr->dat
    stk: Cell;              // stack pointer: relative to base + amxhdr->dat */
    stp: Cell;              // top of the stack: relative to base + amxhdr->dat
    flags: Integer;         // current status, see amx_Flags()
    // user data
    usertags: array[0..AMX_USERNUM-1] of Integer;
    //okay userdata[3] in AMX Mod X is for the CPlugin * pointer
    //we're also gonna set userdata[2] to a special debug structure
    userdata: array[0..AMX_USERNUM-1] of Pointer;
    //native functions can raise an error
    error: Integer;
    // passing parameters requires a "count" field
    ParamCount: Integer;
    // the sleep opcode needs to store the full AMX status
    pri: Cell;
    alt: Cell;
    reset_stk: Cell;
    reset_hea: Cell;
    sysreq_d: Cell;         // relocated address/value for the SYSREQ.D opcode
    // support variables for the JIT
    reloc_size: Integer;    // required temporary buffer for relocations
    code_size: Integer;     // estimated memory footprint of the native code
  end;
  TAMX = tagAMX;

  TAmx_Err_Type = (
    AMX_ERR_NONE,
    // reserve the first 15 error codes for exit codes of the abstract machine */
    AMX_ERR_EXIT,         // forced exit
    AMX_ERR_ASSERT,       // assertion failed
    AMX_ERR_STACKERR,     // stack/heap collision
    AMX_ERR_BOUNDS,       // index out of bounds
    AMX_ERR_MEMACCESS,    // invalid memory access
    AMX_ERR_INVINSTR,     // invalid instruction
    AMX_ERR_STACKLOW,     // stack underflow
    AMX_ERR_HEAPLOW,      // heap underflow
    AMX_ERR_CALLBACK,     // no callback, or invalid callback
    AMX_ERR_NATIVE,       // native function failed
    AMX_ERR_DIVIDE,       // divide by zero
    AMX_ERR_SLEEP,        // go into sleepmode - code can be restarted
    AMX_ERR_INVSTATE,     // invalid state for this access

    AMX_ERR_MEMORY = 16,  // out of memory
    AMX_ERR_FORMAT,       // invalid file format
    AMX_ERR_VERSION,      // file is for a newer version of the AMX
    AMX_ERR_NOTFOUND,     // function not found
    AMX_ERR_INDEX,        // invalid index parameter (bad entry point)
    AMX_ERR_DEBUG,        // debugger cannot run
    AMX_ERR_INIT,         // AMX not initialized (or doubly initialized)
    AMX_ERR_USERDATA,     // unable to set user data field (table full)
    AMX_ERR_INIT_JIT,     // cannot initialize the JIT
    AMX_ERR_PARAMS,       // parameter error
    AMX_ERR_DOMAIN);      // domain error, expression result does not fit in range

  PAMX_NATIVE_INFO = ^AMX_NATIVE_INFO;
  AMX_NATIVE_INFO= packed record
    Name: PChar;
    Func: AMX_NATIVE;      
  end;
  
  TForwardExecType = (
    ET_IGNORE = 0,        // Ignore return vaue
    ET_STOP,              // Stop on PLUGIN_HANDLED
    ET_STOP2,             // Stop on PLUGIN_HANDLED, continue on other values, return biggest return value
    ET_CONTINUE           // Continue; return biggest return value
  );

  TForwardParam = (
    FP_DONE = -1,         // specify this as the last argument
                          // only tells the function that there are no more arguments
    FP_CELL,              // normal cell
    FP_FLOAT,             // float; used as normal cell though
    FP_STRING,            // string
    FP_STRINGEX,          // string; will be updated to the last function's value
    FP_ARRAY              // array; use the return value of prepareArray.
  );

  (*TPlayerProp = (
    Player_Name,          //String
    Player_Ip,            //String
    Player_Team,          //String
    Player_Ingame,        //bool
    Player_Authorized,    //bool
    Player_Vgui,          //bool
    Player_Time,          //float
    Player_Playtime,      //float
    Player_MenuExpire,    //float
    Player_Weapons,       //struct{int,int}[32]
    Player_CurrentWeapon, //int
    Player_TeamID,        //int
    Player_Deaths,        //int
    Player_Aiming,        //int
    Player_Menu,          //int
    Player_Keys,          //int
    Player_Flags,         //int[32]
    Player_Newmenu,       //int
    Player_NewmenuPage    //int
  );*)

  TLibType = (LibType_Library, LibType_Class);

  TErrorInfo = array[0..63] of PChar;

  //---//AUTHORIZEFUNC = procedure(Player: Integer; const AuthString: PChar); cdecl;
  PFN_ADD_NATIVES = function(const List: PAMX_NATIVE_INFO): Integer; cdecl;
  //---//PFN_BUILD_PATHNAME = function(const Format: PChar): PChar; cdecl varargs;
  //---//PFN_BUILD_PATHNAME_R = function(Buffer: PChar; MaxLen: Integer; const Format: PChar): PChar; cdecl varargs;
  PFN_GET_AMXADDR = function(Amx: PAMX; Offset: Cell): PCell; cdecl;
  PFN_PRINT_SRVCONSOLE = procedure(Format: PChar); cdecl varargs;
  PFN_GET_MODNAME = function(): PChar; cdecl;
  //---//PFN_GET_AMXSCRIPTNAME = function(id: Integer): PChar; cdecl;
  //---//PFN_GET_AMXSCRIPT = function(id: Integer): PAMX; cdecl;
  //---//PFN_FIND_AMXSCRIPT_BYAMX = function(Amx: PAMX): Integer; cdecl;
  //---//PFN_FIND_AMXSCRIPT_BYNAME = function(const Name: PChar): Integer; cdecl;
  PFN_SET_AMXSTRING = function(Amx: PAMX; Amx_Addr: Cell; const Source: PChar; Max: Integer): Integer; cdecl;
  PFN_GET_AMXSTRING = function(Amx: PAMX; Amx_Addr: Cell; BufferId: Integer; var Len: Integer): PChar; cdecl;
  //PFN_GET_AMXSTRINGLEN = function(const ptr: PCell): Integer; cdecl;
  PFN_FORMAT_AMXSTRING = function(Amx: PAMX; Params: PCell; StartParam: Integer; var Len: Integer): PChar; cdecl;
  //---//PFN_COPY_AMXMEMORY = procedure(Dest: PCell; const src: PCell; Len: Integer); cdecl;
  PFN_LOG = procedure(const Format: PChar); cdecl varargs;
  PFN_LOG_ERROR = procedure(Amx: PAMX; Err: Integer; const Format: PChar); cdecl varargs;
  ///////PFN_RAISE_AMXERROR = function(Amx: PAMX; Error: Integer): Integer; cdecl;
  //---//PFN_REGISTER_FORWARD = function(const FuncName: PChar; ExecType: TForwardExecType{paramtypes terminated by PF_DONE}): Integer; cdecl varargs;
  //---//PFN_EXECUTE_FORWARD = function(id: Integer): Integer; cdecl varargs;
  //---//PFN_PREPARE_CELLARRAY = function(ptr: PCell; Size: Cardinal): Cell; cdecl;
  //---//PFN_PREPARE_CHARARRAY = function(ptr: PChar; Size: Cardinal): Cell; cdecl;
  //---//PFN_PREPARE_CELLARRAY_A = function(ptr: PCell; Size: Cardinal; CopyBack: Boolean): Cell; cdecl;
  //---//PFN_PREPARE_CHARARRAY_A = function(ptr: PChar; Size: Cardinal; CopyBack: Boolean): Cell; cdecl;
  PFN_IS_PLAYER_VALID = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_NAME = function(id: Integer): PChar; cdecl;
  PFN_GET_PLAYER_IP = function(id: Integer): PChar; cdecl;
  PFN_IS_PLAYER_INGAME = function(id: Integer): Integer; cdecl;
  PFN_IS_PLAYER_BOT = function(id: Integer): Integer; cdecl;
  PFN_IS_PLAYER_AUTHORIZED = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_TIME = function(id: Integer): Single; cdecl;
  PFN_GET_PLAYER_PLAYTIME = function(id: Integer): Single; cdecl;
  PFN_GETPLAYERFLAGS = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_CURWEAPON = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_TEAM = function(id: Integer): PChar; cdecl;
  PFN_GET_PLAYER_TEAMID = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_DEATHS = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_MENU = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_KEYS = function(id: Integer): Integer; cdecl;
  PFN_IS_PLAYER_ALIVE = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_FRAGS = function(id: Integer): Single; cdecl; // function(id: Integer): Integer; cdecl;
  PFN_IS_PLAYER_CONNECTING = function(id: Integer): Integer; cdecl;
  PFN_IS_PLAYER_HLTV = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_ARMOR = function(id: Integer): Integer; cdecl;
  PFN_GET_PLAYER_HEALTH = function(id: Integer): Integer; cdecl;

  //{$ifdef USE_METAMOD}
  //PFN_GET_PLAYER_EDICT = function(id: Integer): PEdict_t; cdecl;
  //{$else}
  PFN_GET_PLAYER_EDICT = function(id: Integer): Pointer; cdecl;
  //{$endif}

  //---//PFN_PLAYER_PROP_ADDR = function(id, Prop: Integer): Pointer; cdecl;

  //{$ifdef MEMORY_TEST}
  //PFN_ALLOCATOR = function(const FileName: PChar; const Line: Cardinal; const Func: PChar; const _Type, Size: Cardinal): Pointer; cdecl;
  //PFN_REALLOCATOR = function(const FileName: PChar; const Line: Cardinal; const Func: PChar; const _Type, Size: Cardinal; Addr: Pointer): Pointer; cdecl;
  //PFN_REALLOCATOR = function(const FileName: PChar; const Line: Cardinal; const Func: PChar; const _Type: Cardinal; const Addr: Pointer): Pointer; cdecl;
  //{$endif}

  //---//PFN_AMX_EXEC = function(Amx: PAMX; ReturnValue: PCell; Index: Integer): Integer; cdecl;
  //---//PFN_AMX_EXECV = function(Amx: PAMX; ReturnValue: PCell; Index, NumParams: Integer; Params: PCell): Integer; cdecl;  //Params: array of Cell
  //---//PFN_AMX_ALLOT = function(Amx: PAMX; Length: Integer; Amx_Addr: PCell; Phys_Addr: PPCell): Integer; cdecl;  // var Phys_Addr: PCell
  //---//PFN_AMX_FINDPUBLIC = function(Amx: PAMX; FuncName: PChar; Index: PInteger): Integer; cdecl;
  //---//PFN_AMX_FINDNATIVE = function(Amx: PAMX; FuncName: PChar; Index: PInteger): Integer; cdecl;
  //---//PFN_LOAD_AMXSCRIPT = function(Amx: PAMX; Code: PPointer; const Path: PChar; ErrorInfo: TErrorInfo; Debug: Integer): Integer; cdecl; //ErrorInfo[64]
  //---//PFN_UNLOAD_AMXSCRIPT = function(Amx: PAMX; Code: PPointer): Integer; cdecl;
  PFN_REAL_TO_CELL = function(X: Single{REAL}): Cell; cdecl;
  //---//PFN_CELL_TO_REAL = function(X: Cell): Single{REAL}; cdecl;
  //---//PFN_REGISTER_SPFORWARD = function(Amx: PAMX; Func: Integer {Params}): Integer; cdecl varargs;
  //---//PFN_REGISTER_SPFORWARD_BYNAME = function(Amx: PAMX; const FuncName: PChar {Params}): Integer; cdecl varargs;
  //---//PFN_UNREGISTER_SPFORWARD = procedure(id: Integer); cdecl;
  //---//PFN_MERGEDEFINITION_FILE = procedure(const FileName: PChar); cdecl;
  //---//PFN_FORMAT = function(const Format: PChar {Params}): PChar; cdecl varargs;
  //---//PFN_REGISTERFUNCTION = procedure(pfn: Pointer; const desc: PChar); cdecl;
  //---//PFN_AMX_PUSH = function(Amx: PAMX; Value: Cell): Integer; cdecl;
  //---//PFN_SET_TEAM_INFO = function(Player, TeamId: Integer; const Name: PChar): Integer; cdecl;
  //---//PFN_REG_AUTH_FUNC = AUTHORIZEFUNC;
  //---//PFN_UNREG_AUTH_FUNC = AUTHORIZEFUNC;
  //---//PFN_FINDLIBRARY = function(const Name: PChar; _Type: TLibType): Integer; cdecl;
  //---//PFN_ADDLIBRARIES = function(const Name: PChar; _Type: TLibType; Parent: Pointer): Cardinal; cdecl;
  //---//PFN_REMOVELIBRARIES = function(Parent: Pointer): Cardinal; cdecl;
  //---//PFN_OVERRIDENATIVES = procedure(Natives: PAMX_NATIVE_INFO; const MyName: PChar); cdecl;
  //---//PFN_GETLOCALINFO = function(const Name, Def: PChar): PChar; cdecl;
  //---//PFN_AMX_REREGISTER = function(Amx: PAMX; List: PAMX_NATIVE_INFO; iList: Integer): Integer; cdecl;
  //---//PFN_REGISTERFUNCTIONEX = function(pfn: Pointer; const desc: PChar): Pointer; cdecl;

  THttpQuery = record
    ClientIP: string;
    Method: string;
    Url: string;
    GetData: string;
    PostData: string;
    Cookies: string;
  end;

  THttpResponse = record
    Code: Integer;
    Location: string;
    Cookies: string;
    ContentType: string;
    Content: string;
  end;

const
  g_ModuleInfo:TAmxx_Module_Info_s = (
    Name: MODULE_NAME;
    Author: MODULE_AUTHOR;
    Version: MODULE_VERSION;
    {$ifdef MODULE_RELOAD_ON_MAPCHANGE}
    Reload: 1;
    {$else}
    Reload: 0;
    {$endif}
    LogTag: MODULE_LOGTAG;
    _Library: MODULE_LIBRARY;
    LibClass: MODULE_LIBCLASS
  );  

var
  g_fn_RequestFunction: PFN_REQ_FNPTR;
  
  g_fn_AddNatives, MF_AddNatives: PFN_ADD_NATIVES;
  //---//g_fn_BuildPathname, MF_BuildPathname: PFN_BUILD_PATHNAME;
  //---//g_fn_BuildPathnameR, MF_BuildPathnameR: PFN_BUILD_PATHNAME_R;
  g_fn_GetAmxAddr, MF_GetAmxAddr: PFN_GET_AMXADDR;
  g_fn_PrintSrvConsole, MF_PrintSrvConsole: PFN_PRINT_SRVCONSOLE;
  g_fn_GetModname, MF_GetModname: PFN_GET_MODNAME;
  //---//g_fn_GetAmxScriptName, MF_GetScriptName: PFN_GET_AMXSCRIPTNAME;
  //---//g_fn_GetAmxScript, MF_GetScriptAmx: PFN_GET_AMXSCRIPT;
  //---//g_fn_FindAmxScriptByAmx, MF_FindScriptByAmx: PFN_FIND_AMXSCRIPT_BYAMX;
  //---//g_fn_FindAmxScriptByName, MF_FindScriptByName: PFN_FIND_AMXSCRIPT_BYNAME;
  g_fn_SetAmxString, MF_SetAmxString: PFN_SET_AMXSTRING;
  g_fn_GetAmxString, MF_GetAmxString: PFN_GET_AMXSTRING;
  //g_fn_GetAmxStringLen, MF_GetAmxStringLen: PFN_GET_AMXSTRINGLEN;
  g_fn_FormatAmxString, MF_FormatAmxString: PFN_FORMAT_AMXSTRING;
  //---//g_fn_CopyAmxMemory, MF_CopyAmxMemory: PFN_COPY_AMXMEMORY;
  g_fn_Log, MF_Log: PFN_LOG;
  g_fn_LogError, MF_LogError: PFN_LOG_ERROR;
  //g_fn_RaiseAmxError, MF_RaiseAmxError: PFN_RAISE_AMXERROR;
  //---//g_fn_RegisterForward, MF_RegisterForward: PFN_REGISTER_FORWARD;
  //---//g_fn_ExecuteForward, MF_ExecuteForward: PFN_EXECUTE_FORWARD;
  //---//g_fn_PrepareCellArray, MF_PrepareCellArray: PFN_PREPARE_CELLARRAY;
  //---//g_fn_PrepareCharArray, MF_PrepareCharArray: PFN_PREPARE_CHARARRAY;
  //---//g_fn_PrepareCellArrayA, MF_PrepareCellArrayA: PFN_PREPARE_CELLARRAY_A;
  //---//g_fn_PrepareCharArrayA, MF_PrepareCharArrayA: PFN_PREPARE_CHARARRAY_A;
  g_fn_IsPlayerValid, MF_IsPlayerValid: PFN_IS_PLAYER_VALID;
  g_fn_GetPlayerName, MF_GetPlayerName: PFN_GET_PLAYER_NAME;
  g_fn_GetPlayerIP, MF_GetPlayerIP: PFN_GET_PLAYER_IP;
  g_fn_IsPlayerIngame, MF_IsPlayerIngame: PFN_IS_PLAYER_INGAME;
  g_fn_IsPlayerBot, MF_IsPlayerBot: PFN_IS_PLAYER_BOT;
  g_fn_IsPlayerAuthorized, MF_IsPlayerAuthorized: PFN_IS_PLAYER_AUTHORIZED;
  g_fn_GetPlayerTime, MF_GetPlayerTime: PFN_GET_PLAYER_TIME;
  g_fn_GetPlayerPlayTime, MF_GetPlayerPlayTime: PFN_GET_PLAYER_PLAYTIME;
  g_fn_GetPlayerFlags, MF_GetPlayerFlags: PFN_GETPLAYERFLAGS;
  g_fn_GetPlayerCurweapon, MF_GetPlayerCurweapon: PFN_GET_PLAYER_CURWEAPON;
  g_fn_GetPlayerTeam, MF_GetPlayerTeam: PFN_GET_PLAYER_TEAM;
  g_fn_GetPlayerTeamID, MF_GetPlayerTeamID: PFN_GET_PLAYER_TEAMID;
  g_fn_GetPlayerDeaths, MF_GetPlayerDeaths: PFN_GET_PLAYER_DEATHS;
  g_fn_GetPlayerMenu, MF_GetPlayerMenu: PFN_GET_PLAYER_MENU;
  g_fn_GetPlayerKeys, MF_GetPlayerKeys: PFN_GET_PLAYER_KEYS;
  g_fn_IsPlayerAlive, MF_IsPlayerAlive: PFN_IS_PLAYER_ALIVE;
  g_fn_GetPlayerFrags, MF_GetPlayerFrags: PFN_GET_PLAYER_FRAGS;
  g_fn_IsPlayerConnecting, MF_IsPlayerConnecting: PFN_IS_PLAYER_CONNECTING;
  g_fn_IsPlayerHLTV, MF_IsPlayerHLTV: PFN_IS_PLAYER_HLTV;
  g_fn_GetPlayerArmor, MF_GetPlayerArmor: PFN_GET_PLAYER_ARMOR;
  g_fn_GetPlayerHealth, MF_GetPlayerHealth: PFN_GET_PLAYER_HEALTH;
  g_fn_GetPlayerEdict, MF_GetPlayerEdict: PFN_GET_PLAYER_EDICT;
  //---//g_fn_AmxExec, MF_AmxExec: PFN_AMX_EXEC;
  //---//g_fn_AmxExecv, MF_AmxExecv: PFN_AMX_EXECV;
  //---//g_fn_AmxAllot, MF_AmxAllot: PFN_AMX_ALLOT;
  //---//g_fn_AmxFindPublic, MF_AmxFindPublic: PFN_AMX_FINDPUBLIC;
  //---//g_fn_LoadAmxScript, MF_LoadAmxScript: PFN_LOAD_AMXSCRIPT;
  //---//g_fn_UnloadAmxScript, MF_UnloadAmxScript: PFN_UNLOAD_AMXSCRIPT;
  g_fn_RealToCell, amx_ftoc : PFN_REAL_TO_CELL;
  //---//g_fn_CellToReal, amx_ctof : PFN_CELL_TO_REAL;
  //---//g_fn_RegisterSPForward, MF_RegisterSPForward: PFN_REGISTER_SPFORWARD;
  //---//g_fn_RegisterSPForwardByName, MF_RegisterSPForwardByName: PFN_REGISTER_SPFORWARD_BYNAME;
  //---//g_fn_UnregisterSPForward, MF_UnregisterSPForward: PFN_UNREGISTER_SPFORWARD;
  //---//g_fn_MergeDefinition_File, MF_MergeDefinition_File: PFN_MERGEDEFINITION_FILE;
  //---//g_fn_AmxFindNative, MF_AmxFindNative: PFN_AMX_FINDNATIVE;
  //---//g_fn_Format, MF_Format: PFN_FORMAT;
  //---//g_fn_RegisterFunction, MF_RegisterFunction, MF_RequestFunction: PFN_REGISTERFUNCTION;
  //---//g_fn_AmxPush, MF_AmxPush: PFN_AMX_PUSH;
  //---//g_fn_SetTeamInfo, MF_SetPlayerTeamInfo: PFN_SET_TEAM_INFO;
  //---//g_fn_PlayerPropAddr, MF_PlayerPropAddr: PFN_PLAYER_PROP_ADDR;
  //---//g_fn_RegAuthFunc, MF_RegAuthFunc: PFN_REG_AUTH_FUNC;
  //---//g_fn_UnregAuthFunc, MF_UnregAuthFunc: PFN_UNREG_AUTH_FUNC;
  //---//g_fn_FindLibrary, MF_FindLibrary: PFN_FINDLIBRARY;
  //---//g_fn_AddLibraries, MF_AddLibraries: PFN_ADDLIBRARIES;
  //---//g_fn_RemoveLibraries, MF_RemoveLibraries: PFN_REMOVELIBRARIES;
  //---//g_fn_OverrideNatives, MF_OverrideNatives: PFN_OVERRIDENATIVES;
  //---//g_fn_GetLocalInfo, MF_GetLocalInfo: PFN_GETLOCALINFO;
  //---//g_fn_AmxReRegister, MF_AmxReRegister: PFN_AMX_REREGISTER;
  //---//g_fn_RegisterFunctionEx, MF_RegisterFunctionEx: PFN_REGISTERFUNCTIONEX;

var
  g_GameRoot: string;
  g_GameModDir: string;
  g_WebMgrRoot: string;
  g_PhpDLLFolder: string;
  g_PhpIniPath: string;
  g_HTTPListenPort: Word;
  g_AdminPassword: string;
  gmsgTextMsg: Integer;

const
  gPlugInfo: TPlugin_Info = (
    ifvers: META_INTERFACE_VERSION;
    name: MODULE_NAME;
    version: MODULE_VERSION;
    date: MODULE_DATE;
    author: MODULE_AUTHOR;
    url: MODULE_URL;
    logtag: MODULE_LOGTAG;
    loadable: Integer(MODULE_LOADABLE);
    unloadable: Integer(MODULE_UNLOADABLE)
  );

  PLID: PPlugin_Info = @gPlugInfo;

procedure GiveFnptrsToDll(pEngFuncsFromEngine: PEngineFuncs; pGlobals: PGlobalVars); stdcall;
function Meta_Detach(now: TPLUG_LOADTIME; reason: TPL_UNLOAD_REASON): Integer; cdecl;
function Meta_Attach(now: TPLUG_LOADTIME; pFunctionTable, pMGlobals, pGamedllFuncs: Pointer): Integer; cdecl;
function Meta_Query(ifvers: PChar; var pPlugInfo: PPlugin_Info; pMetaUtilFuncs: Pointer): Integer; cdecl;

function StringReplaceB(const S, OldPattern, NewPattern: string; Flags: TReplaceFlags): string;

procedure LogPrint(const fmt: string; const Args: array of const);
procedure LogPrintErr(const fmt: string; const Args: array of const);
procedure ConsolePrint(const fmt: string; const Args: array of const);
procedure ConsolePrintErr(const fmt: string; const Args: array of const);

{procedure InitSection;
procedure LockSection;
procedure UnlockSection;
procedure DeleteSection;}

implementation

uses uHttpServer;

{var
  Section: TRTLCriticalSection;

procedure InitSection;
begin
  //InitializeCriticalSection(Section);
end;

procedure LockSection;
begin
  //EnterCriticalSection(Section);
end;

procedure UnlockSection;
begin
  //LeaveCriticalSection(Section);
end;

procedure DeleteSection;
begin
  //DeleteCriticalSection(Section);
end;}

function AMXX_Query(InterfaceVersion: PInteger; ModuleInfo:pamxx_module_info_s): Integer; cdecl;
begin
  // check parameters
  if (InterfaceVersion=nil) or (ModuleInfo=nil) then
  begin
    Result:=AMXX_PARAM;
    Exit;
  end;

  // check interface version
  if InterfaceVersion^ <> AMXX_INTERFACE_VERSION then
  begin
    // Tell amxx core our interface version
    InterfaceVersion^:= AMXX_INTERFACE_VERSION;
    Result:=AMXX_IFVERS;
    Exit;
  end;

  // copy module info
  Move(g_ModuleInfo, ModuleInfo^, SizeOf(TAmxx_Module_Info_s));

  {$ifdef FN_AMXX_QUERY}
  OnAmxxQuery();
  {$endif}

  Result:=AMXX_OK;
end;
    
function AMXX_Attach(reqFnptrFunc: PFN_REQ_FNPTR): Integer; cdecl;
begin
  // Check pointer
  if @reqFnptrFunc=nil then
  begin
    Result:=AMXX_PARAM;
    Exit;
  end;

  @g_fn_RequestFunction:=@reqFnptrFunc;

	// Req all known functions
	// Misc
  //---//@g_fn_BuildPathname:=g_fn_RequestFunction('BuildPathname');
  //---//@g_fn_BuildPathnameR:=g_fn_RequestFunction('BuildPathnameR');
  @g_fn_PrintSrvConsole:=g_fn_RequestFunction('PrintSrvConsole');
  @g_fn_GetModname:=g_fn_RequestFunction('GetModname');
  @g_fn_Log:=g_fn_RequestFunction('Log');
  @g_fn_LogError:=g_fn_RequestFunction('LogError');
  //---//@g_fn_MergeDefinition_File:=g_fn_RequestFunction('MergeDefinitionFile');
  //---//@g_fn_Format:=g_fn_RequestFunction('Format');
  //---//@g_fn_RegisterFunction:=g_fn_RequestFunction('RegisterFunction');

	// Amx scripts
  //---//@g_fn_GetAmxScript:=g_fn_RequestFunction('GetAmxScript');
  //---//@g_fn_FindAmxScriptByAmx:=g_fn_RequestFunction('FindAmxScriptByAmx');
  //---//@g_fn_FindAmxScriptByName:=g_fn_RequestFunction('FindAmxScriptByName');
  //---//@g_fn_LoadAmxScript:=g_fn_RequestFunction('LoadAmxScript');
  //---//@g_fn_UnloadAmxScript:=g_fn_RequestFunction('UnloadAmxScript');
  //---//@g_fn_GetAmxScriptName:=g_fn_RequestFunction('GetAmxScriptName');

  // String / mem in amx scripts support
  @g_fn_SetAmxString:=g_fn_RequestFunction('SetAmxString');
  @g_fn_GetAmxString:=g_fn_RequestFunction('GetAmxString');
  //@g_fn_GetAmxStringLen:=g_fn_RequestFunction('GetAmxStringLen');
  @g_fn_FormatAmxString:=g_fn_RequestFunction('FormatAmxString');
  //---//@g_fn_CopyAmxMemory:=g_fn_RequestFunction('CopyAmxMemory');
  @g_fn_GetAmxAddr:=g_fn_RequestFunction('GetAmxAddr');

  //---//@g_fn_AmxExec:=g_fn_RequestFunction('amx_Exec');
  //---//@g_fn_AmxExecv:=g_fn_RequestFunction('amx_Execv');
  //---//@g_fn_AmxFindPublic:=g_fn_RequestFunction('amx_FindPublic');
  //---//@g_fn_AmxAllot:=g_fn_RequestFunction('amx_Allot');
  //---//@g_fn_AmxFindNative:=g_fn_RequestFunction('amx_FindNative');

	// Natives / Forwards
  @g_fn_AddNatives:=g_fn_RequestFunction('AddNatives');
  //@g_fn_RaiseAmxError:=g_fn_RequestFunction('RaiseAmxError');
  //---//@g_fn_RegisterForward:=g_fn_RequestFunction('RegisterForward');
  //---//@g_fn_RegisterSPForward:=g_fn_RequestFunction('RegisterSPForward');
  //---//@g_fn_RegisterSPForwardByName:=g_fn_RequestFunction('RegisterSPForwardByName');
  //---//@g_fn_UnregisterSPForward:=g_fn_RequestFunction('UnregisterSPForward');
  //---//@g_fn_ExecuteForward:=g_fn_RequestFunction('ExecuteForward');
  //---//@g_fn_PrepareCellArray:=g_fn_RequestFunction('PrepareCellArray');
  //---//@g_fn_PrepareCharArray:=g_fn_RequestFunction('PrepareCharArray');
  //---//@g_fn_PrepareCellArrayA:=g_fn_RequestFunction('PrepareCellArrayA');
  //---//@g_fn_PrepareCharArrayA:=g_fn_RequestFunction('PrepareCharArrayA');
  //---//
  // Player
  @g_fn_IsPlayerValid:=g_fn_RequestFunction('IsPlayerValid');
  @g_fn_GetPlayerName:=g_fn_RequestFunction('GetPlayerName');
  @g_fn_GetPlayerIP:=g_fn_RequestFunction('GetPlayerIP');
  @g_fn_IsPlayerIngame:=g_fn_RequestFunction('IsPlayerInGame');
  @g_fn_IsPlayerBot:=g_fn_RequestFunction('IsPlayerBot');
  @g_fn_IsPlayerAuthorized:=g_fn_RequestFunction('IsPlayerAuthorized');
  @g_fn_GetPlayerTime:=g_fn_RequestFunction('GetPlayerTime');
  @g_fn_GetPlayerPlayTime:=g_fn_RequestFunction('GetPlayerPlayTime');
  @g_fn_GetPlayerFlags:=g_fn_RequestFunction('GetPlayerFlags');
  @g_fn_GetPlayerCurweapon:=g_fn_RequestFunction('GetPlayerCurweapon');
  @g_fn_GetPlayerTeam:=g_fn_RequestFunction('GetPlayerTeam');
  @g_fn_GetPlayerTeamID:=g_fn_RequestFunction('GetPlayerTeamID');
  @g_fn_GetPlayerDeaths:=g_fn_RequestFunction('GetPlayerDeaths');
  @g_fn_GetPlayerMenu:=g_fn_RequestFunction('GetPlayerMenu');
  @g_fn_GetPlayerKeys:=g_fn_RequestFunction('GetPlayerKeys');
  @g_fn_IsPlayerAlive:=g_fn_RequestFunction('IsPlayerAlive');
  @g_fn_GetPlayerFrags:=g_fn_RequestFunction('GetPlayerFrags');
  @g_fn_IsPlayerConnecting:=g_fn_RequestFunction('IsPlayerConnecting');
  @g_fn_IsPlayerHLTV:=g_fn_RequestFunction('IsPlayerHLTV');
  @g_fn_GetPlayerArmor:=g_fn_RequestFunction('GetPlayerArmor');
  @g_fn_GetPlayerHealth:=g_fn_RequestFunction('GetPlayerHealth');
  @g_fn_GetPlayerEdict:=g_fn_RequestFunction('GetPlayerEdict');
  //---//@g_fn_AmxPush:=g_fn_RequestFunction('amx_Push');
  //---//@g_fn_SetTeamInfo:=g_fn_RequestFunction('SetPlayerTeamInfo');
  //---//@g_fn_PlayerPropAddr:=g_fn_RequestFunction('PlayerPropAddr');
  //---//@g_fn_RegAuthFunc:=g_fn_RequestFunction('RegAuthFunc');
  //---//@g_fn_UnregAuthFunc:=g_fn_RequestFunction('UnregAuthFunc');
  //
  //---//@g_fn_FindLibrary:=g_fn_RequestFunction('FindLibrary');
  //---//@g_fn_AddLibraries:=g_fn_RequestFunction('AddLibraries');
  //---//@g_fn_RemoveLibraries:=g_fn_RequestFunction('RemoveLibraries');
  //---//@g_fn_OverrideNatives:=g_fn_RequestFunction('OverrideNatives');

  // Memory
  //{$ifdef MEMORY_TEST}
  //@g_fn_Allocator:=g_fn_RequestFunction('Allocator');
  //@g_fn_Reallocator:=g_fn_RequestFunction('Reallocator');
  //@g_fn_Deallocator:=g_fn_RequestFunction('Deallocator');
  //{$endif}

  //---//@g_fn_CellToReal:=g_fn_RequestFunction('CellToReal');
  @g_fn_RealToCell:=g_fn_RequestFunction('RealToCell');

  
  //
  @MF_AddNatives:=@g_fn_AddNatives;
  //---//@MF_BuildPathname := @g_fn_BuildPathname;
  //---//@MF_BuildPathnameR := @g_fn_BuildPathnameR;
  @MF_FormatAmxString := @g_fn_FormatAmxString;
  @MF_GetAmxAddr := @g_fn_GetAmxAddr;
  @MF_PrintSrvConsole := @g_fn_PrintSrvConsole;
  @MF_GetModname := @g_fn_GetModname;
  //---//@MF_GetScriptName := @g_fn_GetAmxScriptName;
  //---//@MF_GetScriptAmx := @g_fn_GetAmxScript;
  //---//@MF_FindScriptByAmx := @g_fn_FindAmxScriptByAmx;
  //---//@MF_FindScriptByName := @g_fn_FindAmxScriptByName;
  @MF_SetAmxString := @g_fn_SetAmxString;
  @MF_GetAmxString := @g_fn_GetAmxString;
  //@MF_GetAmxStringLen := @g_fn_GetAmxStringLen;
  //---//@MF_CopyAmxMemory := @g_fn_CopyAmxMemory;
  @MF_Log := @g_fn_Log;                     //  C ͬ
  @MF_LogError := @g_fn_LogError;           //  C ͬ
  //@MF_RaiseAmxError := @g_fn_RaiseAmxError;
  //---//@MF_RegisterForward := @g_fn_RegisterForward;
  //---//@MF_ExecuteForward := @g_fn_ExecuteForward;
  //---//@MF_PrepareCellArray := @g_fn_PrepareCellArray;
  //---//@MF_PrepareCharArray := @g_fn_PrepareCharArray;
  //---//@MF_PrepareCellArrayA := @g_fn_PrepareCellArrayA;
  //---//@MF_PrepareCharArrayA := @g_fn_PrepareCharArrayA;
  @MF_IsPlayerValid := @g_fn_IsPlayerValid;
  @MF_GetPlayerName := @g_fn_GetPlayerName;
  @MF_GetPlayerIP := @g_fn_GetPlayerIP;
  @MF_IsPlayerIngame := @g_fn_IsPlayerIngame;
  @MF_IsPlayerBot := @g_fn_IsPlayerBot;
  @MF_IsPlayerAuthorized := @g_fn_IsPlayerAuthorized;
  @MF_GetPlayerTime := @g_fn_GetPlayerTime;
  @MF_GetPlayerPlayTime := @g_fn_GetPlayerPlayTime;
  @MF_GetPlayerFlags := @g_fn_GetPlayerFlags;
  @MF_GetPlayerCurweapon := @g_fn_GetPlayerCurweapon;
  @MF_GetPlayerTeam := @g_fn_GetPlayerTeam;
  @MF_GetPlayerTeamID := @g_fn_GetPlayerTeamID;
  @MF_GetPlayerDeaths := @g_fn_GetPlayerDeaths;
  @MF_GetPlayerMenu := @g_fn_GetPlayerMenu;
  @MF_GetPlayerKeys := @g_fn_GetPlayerKeys;
  @MF_IsPlayerAlive := @g_fn_IsPlayerAlive;
  @MF_GetPlayerFrags := @g_fn_GetPlayerFrags;
  @MF_IsPlayerConnecting := @g_fn_IsPlayerConnecting;
  @MF_IsPlayerHLTV := @g_fn_IsPlayerHLTV;
  @MF_GetPlayerArmor := @g_fn_GetPlayerArmor;
  @MF_GetPlayerHealth := @g_fn_GetPlayerHealth;
  @MF_GetPlayerEdict := @g_fn_GetPlayerEdict;
  //---//@MF_AmxExec := @g_fn_AmxExec;
  //---//@MF_AmxExecv := @g_fn_AmxExecv;
  //---//@MF_AmxFindPublic := @g_fn_AmxFindPublic;
  //---//@MF_AmxAllot := @g_fn_AmxAllot;
  //---//@MF_AmxFindNative := @g_fn_AmxFindNative;
  //---//@MF_LoadAmxScript := @g_fn_LoadAmxScript;
  //---//@MF_UnloadAmxScript := @g_fn_UnloadAmxScript;
  //---//@MF_MergeDefinition_File := @g_fn_MergeDefinition_File;
  //---//@amx_ctof := @g_fn_CellToReal;
  @amx_ftoc := @g_fn_RealToCell;
  //---//@MF_RegisterSPForwardByName := @g_fn_RegisterSPForwardByName;
  //---//@MF_RegisterSPForward := @g_fn_RegisterSPForward;
  //---//@MF_UnregisterSPForward := @g_fn_UnregisterSPForward;
  //---//@MF_Format := @g_fn_Format;
  //---//@MF_RegisterFunction := @g_fn_RegisterFunction;
  //---//@MF_RequestFunction := @g_fn_RequestFunction;
  //---//@MF_AmxPush := @g_fn_AmxPush;
  //---//@MF_SetPlayerTeamInfo := @g_fn_SetTeamInfo;
  //---//@MF_PlayerPropAddr := @g_fn_PlayerPropAddr;
  //---//@MF_RegAuthFunc := @g_fn_RegAuthFunc;
  //---//@MF_UnregAuthFunc := @g_fn_UnregAuthFunc;
  //---//@MF_FindLibrary := @g_fn_FindLibrary;
  //---//@MF_AddLibraries := @g_fn_AddLibraries;
  //---//@MF_RemoveLibraries := @g_fn_RemoveLibraries;
  //---//@MF_OverrideNatives := @g_fn_OverrideNatives;
  //---//@MF_GetLocalInfo := @g_fn_GetLocalInfo;
  //---//@MF_AmxReRegister := @g_fn_AmxReRegister;
  //---//@MF_RegisterFunctionEx := @g_fn_RegisterFunctionEx;

  {$ifdef FN_AMXX_ATTACH}
  OnAmxxAttach();
  {$endif}
  Result:=AMXX_OK;
end;

function AMXX_Detach(): Integer; cdecl;
begin
  {$ifdef FN_AMXX_DETACH}
  OnAmxxDetach();
  {$endif}
  Result:=AMXX_OK;
end;

function AMXX_PluginsLoaded(): Integer; cdecl;
begin
  {$ifdef FN_AMXX_PLUGINSLOADED}
  OnPluginsLoaded();
  {$endif}
  Result:=AMXX_OK;
end;

procedure AMXX_PluginsUnloading(); cdecl;
begin
  {$ifdef FN_AMXX_PLUGINSUNLOADING}
  OnPluginsUnloading();
  {$endif}
end;

procedure AMXX_PluginsUnloaded(); cdecl;
begin
  {$ifdef FN_AMXX_PLUGINSUNLOADED}
  OnPluginsUnloaded();
  {$endif}
end;

{$define LOG_DEBUG}

procedure LogPrint(const fmt: string; const Args: array of const);
var
  S: String;
begin
  S:=Format(fmt, Args);
  MF_Log('[%s] %s', MODULE_LOGTAG, S);
end;

procedure LogPrintErr(const fmt: string; const Args: array of const);
var
  S: String;
begin
  S:=Format(fmt, Args);
  MF_Log('[%s] ERROR: %s', MODULE_LOGTAG, S);
end; 

procedure ConsolePrint(const fmt: string; const Args: array of const);
var
  S: String;
begin
  S:=Format(fmt, Args);
  MF_PrintSrvConsole('[%s] %s'#10, MODULE_LOGTAG, S);
end;

procedure ConsolePrintErr(const fmt: string; const Args: array of const);
var
  S: String;
begin
  S:=Format(fmt, Args);
  MF_PrintSrvConsole('[%s] ERROR: %s'#10, MODULE_LOGTAG, S);
end;

// void WINAPI GiveFnptrsToDll (enginefuncs_t * pengfuncsFromEngine, globalvars_t * pGlobals)
procedure GiveFnptrsToDll(pEngFuncsFromEngine: PEngineFuncs; pGlobals: PGlobalVars); stdcall;
begin
  Move(pEngFuncsFromEngine^, g_EngFuncs, SizeOf(g_EngFuncs));
  gpGlobals:=pGlobals;
end;

// C_DLLEXPORT int Meta_Query (char *ifvers, plugin_info_t ** pPlugInfo, mutil_funcs_t * pMetaUtilFuncs)
function Meta_Query(ifvers: PChar; var pPlugInfo: PPlugin_Info; pMetaUtilFuncs: Pointer): Integer; cdecl;
var
  mmajor, mminor, pmajor, pminor, ColonPos: Integer;
begin
  Result:=0;
  if pMetaUtilFuncs=nil then
  begin
    LogPrintErr('Meta_Query called with null pMetaUtilFuncs', []);
    Exit;
  end;
  gpMetaUtilFuncs:=pMetaUtilFuncs;
  pPlugInfo:=@gPlugInfo;
  if StrPas(ifvers)<>StrPas(gPlugInfo.ifvers) then
  begin
    ColonPos:=Pos(':', StrPas(ifvers));
    mmajor:=StrToIntDef(LeftBStr(ifvers, ColonPos-1), 0);
    mminor:=StrToIntDef(MidBStr(ifvers, ColonPos+1, 10), 0); 
    ColonPos:=Pos(':', StrPas(META_INTERFACE_VERSION));
    pmajor:=StrToIntDef(LeftBStr(META_INTERFACE_VERSION, ColonPos-1), 0);
    pminor:=StrToIntDef(MidBStr(META_INTERFACE_VERSION, ColonPos+1, 10), 0);

    if (pmajor > mmajor) or ((pmajor = mmajor) and (pminor > mminor)) then
    begin
  	  LogPrintErr('metamod version is too old for this plugin; please update metamod.', []);
      Exit;
	  end
    // If plugin has older major interface version, it's incompatible
    // (update plugin).
    else if pmajor < mmajor then
    begin
  	  LogPrintErr('metamod version is incompatible with this plugin; please find a newer version of this plugin.', []);
      Exit;
    end
    // Minor interface is older, but this is guaranteed to be backwards
    // compatible, so we warn, but we still accept it.
    else if (pmajor = mmajor) and (pminor < mminor) then
	    LogPrint('WARNING: metamod version is newer than expected; consider finding a newer version of this plugin.', [])
    else
	    LogPrintErr('unexpected version comparison; metavers=%s, mmajor=%d, mminor=%d; plugvers=%s, pmajor=%d, pminor=%d.',
		    [ifvers, mmajor, mminor, META_INTERFACE_VERSION, pmajor, pminor]);
  end;
  Result:=1;
end;

// C_DLLEXPORT int Meta_Attach (PLUG_LOADTIME now, META_FUNCTIONS * pFunctionTable, meta_globals_t * pMGlobals, gamedll_funcs_t * pGamedllFuncs)
function Meta_Attach(now: TPLUG_LOADTIME; pFunctionTable, pMGlobals, pGamedllFuncs: Pointer): Integer; cdecl;
begin
  Result:=0;
  if now > MODULE_LOADABLE then
  begin
    LogPrintErr('Can''t load plugin right now.', []);
    Exit;
  end;
  Result:=1;
end;

// C_DLLEXPORT int Meta_Detach (PLUG_LOADTIME now, PL_UNLOAD_REASON reason)
function Meta_Detach(now: TPLUG_LOADTIME; reason: TPL_UNLOAD_REASON): Integer; cdecl;
begin
  if (now>MODULE_UNLOADABLE) and (reason<>PNL_CMD_FORCED) then
  begin
    LogPrintErr('Can''t unload plugin right now', []);
    Result:=0;
    Exit;
  end;
  Result:=1;
end;

function StringReplaceB(const S, OldPattern, NewPattern: string;
  Flags: TReplaceFlags): string;
var
  SearchStr, Patt, NewStr: string;
  Offset: Integer;
begin
  if rfIgnoreCase in Flags then
  begin
    SearchStr := UpperCase(S);
    Patt := UpperCase(OldPattern);
  end else
  begin
    SearchStr := S;
    Patt := OldPattern;
  end;
  NewStr := S;
  Result := '';
  while SearchStr <> '' do
  begin
    Offset := Pos(Patt, SearchStr);    
    if Offset = 0 then
    begin
      Result := Result + NewStr;
      Break;
    end;
    Result := Result + Copy(NewStr, 1, Offset - 1) + NewPattern;
    NewStr := Copy(NewStr, Offset + Length(OldPattern), MaxInt);
    if not (rfReplaceAll in Flags) then
    begin
      Result := Result + NewStr;
      Break;
    end;
    SearchStr := Copy(SearchStr, Offset + Length(Patt), MaxInt);
  end;
end;

procedure LOG_DBG_SERVER(S: String);
begin
{$ifdef LOG_DEBUG}
  if S='' then S:='(null)';
  MF_Log('[%s] SERVER: %s', MODULE_LOGTAG, S);
{$endif}
end;

procedure LoadConfig();
var
  MemIni: TMemIniFile;
  ConfigFile: string;
begin
  g_GameRoot:=StringReplace(ExtractFilePath(ParamStr(0)), '\', '/', [rfReplaceAll]);
  g_WebMgrRoot:=g_GameRoot+'webmgr/';
  g_GameModDir:=g_GameRoot+StrPas(MF_GetModname);
  ConfigFile:=g_WebMgrRoot+'configs/webmgr.ini';
  MemIni:=TMemIniFile.Create(ConfigFile);
  try
    MaxHttpReqHeaderSize:=MemIni.ReadInteger('webmgr', 'Max_Http_Req_Header_Size', MaxHttpReqHeaderSize);
    MaxHttpReqContentLength:=MemIni.ReadInteger('webmgr', 'Max_Http_Req_Content_Length', MaxHttpReqContentLength);
    g_PhpDLLFolder:=UTF8Decode(MemIni.ReadString('webmgr', 'PHP_Dll_Folder', './webmgr/php-5.3.0-Win32-VC6-x86/'));
    g_PhpIniPath:=UTF8Decode(MemIni.ReadString('webmgr', 'PHP_Ini_Path', './webmgr/php-5.3.0-Win32-VC6-x86/php-webmgr.ini'));
    g_HTTPListenPort:=MemIni.ReadInteger('webmgr', 'HTTP_Listen_Port', 0);
    g_AdminPassword:=MemIni.ReadString('webmgr', 'Admin_Password', '');
  finally
    MemIni.Free;
  end;
end;

procedure OnAmxxAttach();
begin
  MF_PrintSrvConsole(#10);
  dtStartTime:=Now;
  g_EngFuncs.pfnCVarRegister(@MODULE_CVAR_VERSION);
  LoadConfig();
  dmHttpServer:=TdmHttpServer.Create(nil);
  PHPVersion:=GetPHPVersion();
  MF_PrintSrvConsole(#10);
end;

exports
  // Meta functions
  GiveFnptrsToDll,
  Meta_Query,
  Meta_Attach,
  Meta_Detach,
  // AMXX functions
  AMXX_Query,
  AMXX_Attach,
  AMXX_Detach,
  AMXX_PluginsLoaded,
  AMXX_PluginsUnloaded,
  AMXX_PluginsUnloading;

end.
