unit uHttpServer;

interface

uses
  SysUtils, Classes, Windows, php4delphi, PHPCustomLibrary, phpLibrary, PHPCommon,
  phpFunctions, StrUtils, zendAPI, Registry, Variants, PHPTypes,
  PHPAPI, Winsock, zendTypes, DateUtils, IniFiles;

type
  TMyPsvPHP = class(TpsvPHP)
  private
    { Private declarations }
  public
    RequestClientIP: string;
    constructor Create(AOwner : TComponent); override;
  end;

  PVirtualDir = ^TVirtualDir;
  TVirtualDir = record
    VDir: string;
    Path: string;
    DenyExt: string;
  end;

  TConnectionThread = class(TThread)
  private
    ConnSocket: TSocket;
    PHP: TMyPsvPHP;
    FSended: Boolean;
    
    RequestHeader: string;
    
    RequestMethod: string;
    RequestUrl: string;
    RequestGetData: string;
    RequestCookies: string; 
    RequestPostData: string;
    //RequestClientIP: string;
    RequestContentType: string;

    ResponseCode: Integer;
    ResponseLocation: string;
    ResponseContentType: string;
    ResponseContent: string;

    procedure ProcessRequest;
    procedure SendHeaders;
    procedure SendResponse;
    procedure SendFile(FileName: string);
    procedure RegisterHttpVariables;
    procedure PHPReadPost(Sender: TObject; Stream: TStream);
    procedure PHPReadResult(Sender: TObject; Stream: TStream);
  protected
    procedure Execute; override;
  public
    ClientIP: string;
    constructor Create(Socket: TSocket);
  end;
  
  TdmHttpServer = class(TDataModule)
    PHPEngine: TPHPEngine;
    PHPLib: TPHPLibrary;
    procedure DataModuleCreate(Sender: TObject);
    procedure PHPLibFunctions0Execute(Sender: TObject;
      Parameters: TFunctionParams; var ReturnValue: Variant;
      ZendVar: TZendVariable; TSRMLS_DC: Pointer);
    procedure DataModuleDestroy(Sender: TObject);
    procedure PHPEngineLogMessage(Sender: TObject; AText: String);
  private
    ServerSocket: TSocket;
  end;

var
  dmHttpServer: TdmHttpServer;
  PHPVersion: TPHPFileInfo;
  dtStartTime: TDateTime;
  VDirList: TList;

function GetServerSoftware: string;

implementation

uses uMetaAPI, uAmxxModule;

{$R *.dfm}

{function GetOutAddr(hSocket: TSocket; szTargetAddr: string): string;
var
  dwBytesReturned: DWORD;  
	saTarget, saSource: TSockAddrIn ;
begin
	dwBytesReturned:=0;
	ZeroMemory(@saSource, SizeOf(saSource));
	ZeroMemory(@saTarget, SizeOf(saTarget));
	saTarget.sin_family:=AF_INET;
  saSource.sin_family:=AF_INET;
	saTarget.sin_addr.s_addr:=inet_addr(PChar(szTargetAddr));
	WSAIoctl(hSocket, SIO_ROUTING_INTERFACE_QUERY, @saTarget, sizeof(sockaddr_in), @saSource, sizeof(sockaddr_in), @dwBytesReturned, nil, nil);
	Result:=inet_ntoa(saSource.sin_addr);
end;} 

function HexToChar(Hex: PChar): Char;
begin
  Result:=' ';
  HexToBin(Hex, @Result, 1);
end;

function UrlDecode(Url: string): string;
var
  Len, I: Integer;
  C: Char;
begin
  Len:=Length(Url);
  I:=1;
  while I<=Len do
  begin
    C:=Url[I];
    case C of
      '+': Result:=Result+' ';
      '%':
      begin
        if I>Len-2 then Break;  // incomplete
        Result:=Result+string(HexToChar(@Url[I+1]));
        Inc(I, 2);
      end;
    else
      Result:=Result+string(C);
    end;
    Inc(I);
  end;
end;

function GetServerSoftware: string;
begin
  Result:=Format('WebMgr/%s (Win32) PHP/%d.%d.%d',
    [MODULE_VERSION, PHPVersion.MajorVersion, PHPVersion.MinorVersion, PHPVersion.Release]);
end;

function GetUsersNum(IncludeBots: Boolean): Integer;
var
  I: Integer;
begin
  Result:=0;
  for I := 1 to gpGlobals.maxClients do
  begin
    if Boolean(MF_IsPlayerValid(I)) then
    begin
      if IncludeBots or (not Boolean(MF_IsPlayerBot(I))) then
        Inc(Result);
    end;
  end;
end;

procedure GetAllMaps(pval: pzval);
var
  MapName: string;
  iFind: Integer;
  SearchRec: TSearchRec;
begin
  iFind:=SysUtils.FindFirst(g_GameModDir+'/maps/*.bsp', faAnyFile and not faDirectory, SearchRec);
  if iFind=0 then
  begin
    try
      repeat
        MapName:=ChangeFileExt(SearchRec.Name,'');
        if Boolean(g_EngFuncs.pfnIsMapValid(PChar(MapName))) then
          add_next_index_string(pval, PChar(MapName), 1);
      until SysUtils.FindNext(SearchRec)<>0;
    finally
      SysUtils.FindClose(SearchRec);
    end;
  end;
end;

procedure GetCycleMaps(pval: pzval);
var
  CycleFile: string;
  SL: TStringList;
  sLine: string;
  I: Integer;
begin
  CycleFile:=g_GameModDir+'/'+StrPas(g_EngFuncs.pfnCVarGetString('mapcyclefile'));
  SL:=TStringList.Create;
  try
    try
      SL.LoadFromFile(CycleFile);
    except
      Exit;
    end;
    for I := 0 to SL.Count - 1 do
    begin
      sLine:=Trim(SL.Strings[I]);
      if sLine='' then Continue;
      if Boolean(g_EngFuncs.pfnIsMapValid(PChar(sLine))) then
        add_next_index_string(pval, PChar(sLine), 1);
    end;
  finally
    SL.Free;
  end;
end;

procedure UTIL_ClientPrint(pEntity: Pointer; msg_dest: Integer; msg: string);
begin
  if gmsgTextMsg=0 then
    gmsgTextMsg:=gpMetaUtilFuncs.pfnGetUserMsgID(PLID, 'TextMsg', nil);
	if gmsgTextMsg=0 then Exit;
  if Length(msg)>126 then     // max msg size is 127
  begin
    msg[127]:=#10;
    msg[128]:=#0;
  end else
    msg:=msg+#10;
	if pEntity<>nil then
		g_EngFuncs.pfnMessageBegin(MSG_ONE, gmsgTextMsg, nil, pEntity)
	else
		g_EngFuncs.pfnMessageBegin(MSG_BROADCAST, gmsgTextMsg);
	g_EngFuncs.pfnWriteByte(msg_dest);
	g_EngFuncs.pfnWriteString(PChar(msg));
	g_EngFuncs.pfnMessageEnd();
end;

{procedure SaveToLog(Text: string);
var
  Year, Month, Day, Hour, Minute, Second: Integer;
  CurTime: TDateTime;
  FileName: string;
begin
  CurTime:=Now();
  Year:=YearOf(CurTime);
  Month:=MonthOf(CurTime);
  Day:=DayOf(CurTime);
  Hour:=HourOf(CurTime);
  Minute:=MinuteOf(CurTime);
  Second:=SecondOf(CurTime);
  Text:=Format('[%0.4d-%0.2d-%0.2d %0.2d:%0.2d:%0.2d] %s',
    [Year, Month, Day, Hour, Minute, Second, Text]);
  FileName:=g_WebMgrRoot+'logs/'+Format('%0.4d-%0.2d.log', [Year, Month]);
end;}

function getAddr: TSockAddrIn;
var
  ip: PChar;
  Port: Word;
  he: PHostEnt;
  net_addr: string;
  ColonPos: Integer;
begin
  ip:=g_EngFuncs.pfnCVarGetString('ip');
  if CompareText(ip,'localhost')=0 then
    Result.sin_addr.s_addr:=INADDR_ANY
  else begin
    Result.sin_addr.s_addr:=inet_addr(ip);
    if Integer(Result.sin_addr.s_addr) = INADDR_NONE then
    begin
			he:=gethostbyname(ip);
      if he<>nil then
        Result.sin_addr.s_addr:=PInteger(he.h_addr_list^)^;
      if Integer(Result.sin_addr.s_addr) = INADDR_NONE then
      begin
        net_addr:=StrPas(g_engfuncs.pfnCVarGetString('net_address'));
        ColonPos:=Pos(':', net_addr);
        if ColonPos>0 then
        begin
          net_addr[ColonPos]:=#0;
          Result.sin_addr.s_addr:=inet_addr(PChar(net_addr));
        end;
        if Integer(Result.sin_addr.s_addr) = INADDR_NONE then
          Result.sin_addr.s_addr:=INADDR_ANY;
      end;
    end;
  end;

  Port:=g_HTTPListenPort;
	if Port=0 then
  begin
	  Port:=Trunc(g_engfuncs.pfnCVarGetFloat('hostport'));
  	if Port=0 then
    begin
	  	Port:=Trunc(g_engfuncs.pfnCVarGetFloat('port'));
      if Port=0 then
      begin
        net_addr:=StrPas(g_engfuncs.pfnCVarGetString('net_address'));
        ColonPos:=Pos(':', net_addr);
        if ColonPos>0 then
          Port:=StrToIntDef(MidBStr(net_addr, ColonPos+1, 5), 27015);
        if Port=0 then Port:=27015;        
      end;
    end;
  end;
	Result.sin_family:=AF_INET;
	Result.sin_port:= htons(Port);
  ConsolePrint('Listening on %s:%u', [inet_ntoa(Result.sin_addr), Port]);
end;

function waitInput(ConnSocket: TSocket; Secs: Integer): Integer;
var
  rfds: TFDSet;
  tv: TTimeVal;
begin
	FD_ZERO(rfds);
	FD_SET(ConnSocket, rfds);
	tv.tv_sec:=Secs;
	tv.tv_usec:=0;
	Result:=select(1, @rfds, nil, nil, @tv);
end;

function ReceiveText(Socket: TSocket): string;
var
  re: Integer;
begin
  SetLength(Result, 8192);
  re:=recv(Socket, PChar(Result)^, 8192, 0);
  if re>0 then
    SetLength(Result, re)
  else
    Result:='';
end;

function WaitingConnection(Sock: Pointer): DWORD; stdcall;
var
  ServerSocket, ConnSocket: TSocket;
  SockLen: Integer;
  Sin: TSockAddrIn;
  Thread: TConnectionThread;
begin
  ServerSocket:=TSocket(Sock);
  while True do
  begin
    SockLen:=SizeOf(TSockAddrIn);
	  ConnSocket:=accept(ServerSocket, @Sin, @SockLen);
		if ConnSocket=INVALID_SOCKET then
		begin
			LogPrintErr('Error while accepting connection.', []);
			Continue;
		end;
    Thread:=TConnectionThread.Create(ConnSocket);
    Thread.ClientIP:=StrPas(inet_ntoa(Sin.sin_addr));
    Thread.Resume;
  end;
  Result:=0;  // Never happens...
end;

constructor TMyPsvPHP.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  RequestClientIP:='';
end;

procedure TdmHttpServer.DataModuleCreate(Sender: TObject);
var
  Sin: TSockAddrin;
  ThreadHandle: THandle;
  ThreadID: Cardinal;
  ConfigFile, VDir, Path, DenyExt: string;
  MemIni: TMemIniFile;
  Sections: TStringList;
  I: Integer;
  PVDir: PVirtualDir;
begin
  PHPEngine.DLLFolder:=g_PhpDLLFolder;
  PHPEngine.IniPath:=g_PhpIniPath;
  try
    PHPEngine.StartupEngine;
  except
    ConsolePrintErr('Could not Startup PHP Engine.', []);
    Exit;
  end;
  VDirList:=TList.Create;
  // set "/" as first virtual directory
  New(PVDir);
  PVDir^.VDir:='/';
  PVDir^.Path:=g_WebMgrRoot+'www/';
  PVDir^.DenyExt:='';
  VDirList.Add(PVDir);
  //
  ConfigFile:=g_WebMgrRoot+'configs/vdir.ini';
  MemIni:=TMemIniFile.Create(ConfigFile);
  try
    Sections:=TStringList.Create;
    try
      MemIni.ReadSections(Sections);
      for I := 0 to Sections.Count - 1 do
      begin
        VDir:=Trim(Sections.Strings[I]);
        Path:=Trim(MemIni.ReadString(VDir, 'Path', ''));
        DenyExt:=Trim(MemIni.ReadString(VDir, 'DenyExt', '')); 
        Path:=StringReplaceB(Path, '\', '/', [rfReplaceAll]);
        if (VDir<>'') and (Path<>'') then
        begin
          VDir:=StringReplaceB(VDir, '\', '/', [rfReplaceAll]);
          if VDir[1]<>'/' then
            VDir:='/'+VDir;
          if VDir[Length(VDir)]<>'/' then
            VDir:=VDir+'/';
          if Path[Length(Path)]<>'/' then
            Path:=Path+'/';
          New(PVDir);
          PVDir^.VDir:=VDir;
          PVDir^.Path:=UTF8Decode(Path);
          PVDir^.DenyExt:=DenyExt;
          VDirList.Add(PVDir);
        end;
      end;
    finally
      Sections.Free;
    end;
  finally
    MemIni.Free;
  end;
  sin:=getAddr;
  ServerSocket:=socket(AF_INET,SOCK_STREAM,0);
  if ServerSocket=INVALID_SOCKET then
  begin
    ConsolePrintErr('Could not create socket.', []);
    Exit;
  end;
  setsockopt(ServerSocket, SOL_SOCKET, SO_REUSEADDR, #$FF#$FF#$FF#$FF, 4);
  //bind(ServerSocket, sin, sizeof(sin));
  if bind(ServerSocket, sin, sizeof(sin))=SOCKET_ERROR then
  begin
    ConsolePrintErr('Could not bind socket to ip %s:%d.', [inet_ntoa(sin.sin_addr), ntohs(sin.sin_port)]);
    Exit;
  end;
  if listen(ServerSocket, 1024)=SOCKET_ERROR then
  begin
    ConsolePrintErr('Could not make the socket listen.', []);
    Exit;
  end;
  ThreadHandle:=CreateThread(nil, 0, @WaitingConnection, Pointer(ServerSocket), CREATE_SUSPENDED, ThreadID);
  if ThreadHandle=0then
  begin
    ConsolePrintErr('Failure starting thread "WaitingConnection".', []);
    Exit;
  end;
  try
    SetThreadPriority(ThreadHandle, THREAD_PRIORITY_IDLE);
    if ResumeThread(ThreadHandle)=DWORD(-1) then
    begin
      ConsolePrintErr('Failure resume thread "WaitingConnection".', []);
      Exit;
    end;
  finally
    CloseHandle(ThreadHandle);  
  end;
	ConsolePrint('Now ready: listening...', []);
end;

procedure TdmHttpServer.DataModuleDestroy(Sender: TObject);
var
  I: Integer;
  PVDir: PVirtualDir;
begin
  for I := 0 to VDirList.Count - 1 do
  begin
    PVDir:=VDirList.Items[I];
    Dispose(PVDir);
  end;
  VDirList.Free;
  try
    PHPEngine.ShutdownEngine;// .ShutdownAndWaitFor;
  except
  end;
end;

procedure TdmHttpServer.PHPEngineLogMessage(Sender: TObject;
  AText: String);
begin
  // Do nothing, used to avoid show message box
end;

procedure TdmHttpServer.PHPLibFunctions0Execute(Sender: TObject;
  Parameters: TFunctionParams; var ReturnValue: Variant;
  ZendVar: TZendVariable; TSRMLS_DC: Pointer);
var
  FunctionName, sIP: string;
  iPos, iPing, iLoss: Integer;
  PHP: TMyPsvPHP;
  gl : psapi_globals_struct;
  pval : pzval;
begin
  ReturnValue:='';
  gl := GetSAPIGlobals;
  if gl = nil then  Exit;
  if not (TObject(gl^.server_context) is TMyPsvPHP) then Exit;    
  PHP:=TMyPsvPHP(gl^.server_context);
  try
    FunctionName := LowerCase(get_active_function_name(TSRMLS_DC));
    if FunctionName='webmgr_version' then
      ReturnValue:=string(MODULE_VERSION)
    else if FunctionName='webmgr_elapsed_time' then
      ReturnValue:=SecondSpan(Now,dtStartTime)
    else if FunctionName='webmgr_get_mod_path' then
      ReturnValue:=g_GameModDir+'/'
    else if FunctionName='webmgr_get_cvar' then
      ReturnValue:=StrPas(g_EngFuncs.pfnCVarGetString(PChar(string(Parameters[0].Value))))
    else if FunctionName='webmgr_set_cvar' then
      g_EngFuncs.pfnCVarSetString(PChar(string(Parameters[0].Value)), PChar(string(Parameters[1].Value)))
    else if FunctionName='webmgr_is_map_valid' then
      ReturnValue:=Boolean(g_EngFuncs.pfnIsMapValid(PChar(string(Parameters[0].Value))))
    else if FunctionName='webmgr_get_mapname' then
        ReturnValue:=StrPas(_STRING(gpGlobals.mapname))
    else if FunctionName='webmgr_changelevel' then
    begin
      if Boolean(g_EngFuncs.pfnIsMapValid(PChar(string(Parameters[0].Value)))) then
      begin
        LogPrint('Change level: "%s". Client IP: %s', [string(Parameters[0].Value), PHP.RequestClientIP]);
        g_EngFuncs.pfnServerCommand(PChar(Format('changelevel "%s"'#10, [string(Parameters[0].Value)])));
      end;
    end
    else if FunctionName='webmgr_get_allmaps' then
    begin
      pval := ZendVar.AsZendVariable;
      if _array_init(pval, nil, 0) = FAILURE then
      begin
        php_error_docref(nil , TSRMLS_DC, E_ERROR, 'Unable to initialize array');
        ZVAL_FALSE(pval);
        Exit;
      end;
      GetAllMaps(pval);
    end
    else if FunctionName='webmgr_get_cyclemaps' then
    begin
      pval := ZendVar.AsZendVariable;
      if _array_init(pval, nil, 0) = FAILURE then
      begin
        php_error_docref(nil , TSRMLS_DC, E_ERROR, 'Unable to initialize array');
        ZVAL_FALSE(pval);
        Exit;
      end;
      GetCycleMaps(pval);
    end
    else if FunctionName='webmgr_is_player_valid' then
      ReturnValue:=Boolean(MF_IsPlayerValid(Parameters[0].Value))
    else if FunctionName='webmgr_is_player_connected' then
      ReturnValue:=Boolean(MF_IsPlayerInGame(Parameters[0].Value))
    else if FunctionName='webmgr_is_player_connecting' then
      ReturnValue:=Boolean(MF_IsPlayerConnecting(Parameters[0].Value))
    else if FunctionName='webmgr_is_player_bot' then
      ReturnValue:=Boolean(MF_IsPlayerBot(Parameters[0].Value))
    else if FunctionName='webmgr_is_player_hltv' then
      ReturnValue:=Boolean(MF_IsPlayerHLTV(Parameters[0].Value))
    else if FunctionName='webmgr_is_player_alive' then
      ReturnValue:=Boolean(MF_IsPlayerAlive(Parameters[0].Value))
    else if FunctionName='webmgr_get_player_name' then
      ReturnValue:=StrPas(MF_GetPlayerName(Parameters[0].Value))
    else if FunctionName='webmgr_get_player_ip' then 
    begin
      sIP:=StrPas(MF_GetPlayerIp(Parameters[0].Value));
      iPos:=Pos(':', sIP);
      if iPos<=0 then
        ReturnValue:=sIP
      else
        ReturnValue:=LeftBStr(sIP, iPos-1);
    end
    else if FunctionName='webmgr_get_player_time' then
      ReturnValue:=gpGlobals.time-MF_GetPlayerTime(Parameters[0].Value)
    else if FunctionName='webmgr_get_player_playtime' then
    begin
      if Boolean(MF_IsPlayerInGame(Parameters[0].Value)) then
        ReturnValue:=gpGlobals.time-MF_GetPlayerPlayTime(Parameters[0].Value)
      else
        ReturnValue:=Integer(0);
    end
    else if FunctionName='webmgr_get_player_team' then
      ReturnValue:=StrPas(MF_GetPlayerTeam(Parameters[0].Value))
    else if FunctionName='webmgr_get_player_teamid' then
    begin
      if not Boolean(MF_IsPlayerInGame(Parameters[0].Value)) then
        ReturnValue:=Integer(-1)
      else
        ReturnValue:=MF_GetPlayerTeamID(Parameters[0].Value);
    end
    else if FunctionName='webmgr_get_player_frags' then
      ReturnValue:=Integer(Trunc(MF_GetPlayerFrags(Parameters[0].Value)))
    else if FunctionName='webmgr_get_player_deaths' then
      ReturnValue:=MF_GetPlayerDeaths(Parameters[0].Value)
    else if FunctionName='webmgr_get_player_ping' then
    begin
      if not Boolean(MF_IsPlayerInGame(Parameters[0].Value)) then
        ReturnValue:=Integer(0)
      else begin
        g_EngFuncs.pfnGetPlayerStats(MF_GetPlayerEdict(Parameters[0].Value), iPing, iLoss);
        ReturnValue:=iPing;
      end;
    end
    else if FunctionName='webmgr_get_player_userid' then
    begin
      if not Boolean(MF_IsPlayerValid(Parameters[0].Value)) then
        ReturnValue:=Integer(0)
      else
        ReturnValue:=g_EngFuncs.pfnGetPlayerUserId(MF_GetPlayerEdict(Parameters[0].Value));
    end
    else if FunctionName='webmgr_get_player_authid' then
    begin
      if not Boolean(MF_IsPlayerValid(Parameters[0].Value)) then
        ReturnValue:='UNKNOWN'
      else
        ReturnValue:=StrPas(g_EngFuncs.pfnGetPlayerAuthId(MF_GetPlayerEdict(Parameters[0].Value)));
    end
    else if FunctionName='webmgr_server_cmd' then
    begin
      LogPrint('Excute server command: "%s". Client IP: %s', [string(Parameters[0].Value), PHP.RequestClientIP]);
      g_EngFuncs.pfnServerCommand(PChar(string(Parameters[0].Value)+#10));
    end
    else if FunctionName='webmgr_server_exec' then
      g_EngFuncs.pfnServerExecute
    else if FunctionName='webmgr_client_cmd' then
    begin
      if Boolean(MF_IsPlayerValid(Parameters[0].Value)) then
      begin
        LogPrint('Execute client command: "%s" to "%s<%d><%s><>". Client IP: %s',
          [string(Parameters[1].Value), MF_GetPlayerName(Parameters[0].Value),
          g_EngFuncs.pfnGetPlayerUserId(MF_GetPlayerEdict(Parameters[0].Value)),
          g_EngFuncs.pfnGetPlayerAuthId(MF_GetPlayerEdict(Parameters[0].Value)),
          PHP.RequestClientIP]);
        g_EngFuncs.pfnClientCommand(MF_GetPlayerEdict(Parameters[0].Value), '%s', PChar(string(Parameters[1].Value)+#10));
      end;
    end 
    else if FunctionName='webmgr_client_print' then
    begin
      if Boolean(MF_IsPlayerValid(Parameters[0].Value)) then
      begin
        LogPrint('Print client message: "%s" to "%s<%d><%s><>" (Print Type: %d). Client IP: %s',
          [string(Parameters[2].Value), MF_GetPlayerName(Parameters[0].Value),
          g_EngFuncs.pfnGetPlayerUserId(MF_GetPlayerEdict(Parameters[0].Value)),
          g_EngFuncs.pfnGetPlayerAuthId(MF_GetPlayerEdict(Parameters[0].Value)),
          Integer(Parameters[1].Value), PHP.RequestClientIP]);
        UTIL_ClientPrint(MF_GetPlayerEdict(Parameters[0].Value), Parameters[1].Value, Parameters[2].Value);
      end;
    end
    else if FunctionName='webmgr_get_admin_password' then
      ReturnValue:=g_AdminPassword
    else if FunctionName='webmgr_get_maxplayers' then
      ReturnValue:=gpGlobals.maxClients
    else if FunctionName='webmgr_get_usersnum' then
      ReturnValue:=GetUsersNum(False)
    else if FunctionName='webmgr_get_playersnum' then
      ReturnValue:=GetUsersNum(True);
  except
  end;
end;

constructor TConnectionThread.Create(Socket: TSocket);
begin
  inherited Create(True);
  ConnSocket:=Socket;
  FSended:=False;
  FreeOnTerminate:=True;
  Priority:=tpIdle;
end;

procedure TConnectionThread.Execute;
var
  RequestText, RequestHeaderLC, RecvText: string;
  QuestionMarkPos, SpacePos, ClPos, LfPos, DLfPos, CtPos, CookiePos: Integer;
  ContentLength, I: Integer;
  BreakFlag: Boolean;
begin
  PHP:=TMyPsvPHP.Create(nil);
  try
    PHP.RequestClientIP:=ClientIP;
    RequestText:='';
    DLfPos:=0;
    BreakFlag:=True;
    while True do
    begin
      if Length(RequestText)>MaxHttpReqHeaderSize then Exit; // too big header size
      if waitInput(ConnSocket, 5)<1 then
        Exit;     // receive time out
      RecvText:=ReceiveText(ConnSocket);
      if RecvText='' then Exit;  // receive error
      RequestText:=RequestText+RecvText;
      DLfPos:=Pos(#13#10#13#10, RequestText);
      if DLfPos>0 then Break;
      DLfPos:=Pos(#10#10, RequestText);
      if DlfPos>0 then
      begin
        BreakFlag:=False;
        Break;
      end;
    end;
    RequestHeader:=StringReplaceB(LeftBStr(RequestText, DLfPos-1), #13, '', [rfReplaceAll])+#10;
    // check RequestHeader is standard ansi string
    for I := 1 to Length(RequestHeader) do
    begin
      if ((RequestHeader[I]<#32) and (RequestHeader[I]<>#10)) or (RequestHeader[I]>#127) then
        Exit;      
    end;
    RequestHeaderLC:=LowerCase(RequestHeader);
    if BreakFlag then
      RequestPostData:=MidBStr(RequestText, DLfPos+4, Length(RequestText))
    else
      RequestPostData:=MidBStr(RequestText, DLfPos+2, Length(RequestText));
    LfPos:=Pos(#10, RequestHeader);
    if LeftBStr(RequestHeaderLC, 4)='get ' then
    begin
      RequestMethod:='GET';
      QuestionMarkPos:=PosEx('?', RequestHeader, 5);
      if (5<QuestionMarkPos) and (QuestionMarkPos<LfPos) then
      begin
        RequestUrl:=MidBStr(RequestHeader, 5, QuestionMarkPos-5);
        SpacePos:=PosEx(' ', RequestHeader, QuestionMarkPos);
        if (SpacePos<=QuestionMarkPos) or (SpacePos>LfPos) then Exit;
        RequestGetData:=MidBStr(RequestHeader, QuestionMarkPos+1, SpacePos-QuestionMarkPos-1);
      end else begin
        SpacePos:=PosEx(' ', RequestHeader, 5);
        if (SpacePos<=5) or (SpacePos>LfPos) then Exit;
        RequestUrl:=MidBStr(RequestHeader, 5, SpacePos-5);
      end;
    end else if LeftBStr(RequestHeaderLC, 5)='post ' then begin
      RequestMethod:='POST';
      QuestionMarkPos:=PosEx('?', RequestHeader, 6);
      if (6<QuestionMarkPos) and (QuestionMarkPos<LfPos) then
      begin
        RequestUrl:=MidBStr(RequestHeader, 6, QuestionMarkPos-6);
        SpacePos:=PosEx(' ', RequestHeader, QuestionMarkPos);
        if (SpacePos<=QuestionMarkPos) or (SpacePos>LfPos) then Exit;
        RequestGetData:=MidBStr(RequestHeader, QuestionMarkPos+1, SpacePos-QuestionMarkPos-1);
      end else begin
        SpacePos:=PosEx(' ', RequestHeader, 6);
        if (SpacePos<=6) or (SpacePos>LfPos) then Exit;
        RequestUrl:=MidBStr(RequestHeader, 6, SpacePos-6);
      end;
      ClPos:=Pos(#10'content-length: ', RequestHeaderLC);
      if ClPos>0 then
      begin
        LfPos:=PosEx(#10, RequestHeader, ClPos+17);
        try
          ContentLength:=StrToInt(MidBStr(RequestHeader, ClPos+17, LfPos-ClPos-17));
        except
          Exit;  // bad content length
        end;
        if (ContentLength<0) or (ContentLength>MaxHttpReqContentLength) then  // bad content length OR too big content length
          Exit;
        while Length(RequestPostData)<ContentLength do
        begin
          if waitInput(ConnSocket, 5)<1 then Exit;  // receive time out
          RecvText:=ReceiveText(ConnSocket);
          if RecvText='' then
            Exit;  // receive error
          RequestPostData:=RequestPostData+RecvText;
        end;
      end;
    end else begin
      //Exit;
    end;
    // Process cookies
    CookiePos:=Pos(#10'cookie: ', RequestHeaderLC);
    while CookiePos>0 do
    begin
      LfPos:=PosEx(#10, RequestHeader, CookiePos+10);
      if RequestCookies='' then
        RequestCookies:=MidBStr(RequestHeader, CookiePos+9, LfPos-CookiePos-9)
      else
        RequestCookies:=RequestCookies+'; '+MidBStr(RequestHeader, CookiePos+9, LfPos-CookiePos-9);
      CookiePos:=PosEx(#10'cookie: ', RequestHeaderLC, LfPos);
    end;
    // Process Content type 
    CtPos:=Pos(#10'content-type: ', RequestHeaderLC);
    if CtPos>0 then
    begin
      LfPos:=PosEx(#10, RequestHeader, CtPos+15);
      RequestContentType:=MidBStr(RequestHeader, CtPos+15, LfPos-CtPos-15);
    end;
    ProcessRequest();
  finally
    PHP.Free;
    shutdown(ConnSocket, SD_BOTH);
    closesocket(ConnSocket);
  end;
end;

procedure TConnectionThread.RegisterHttpVariables;
var
  sLine, sField: string;
  PosLf, PosLf2, PosColon: Integer;
begin
  PosLf:=Pos(#10, RequestHeader);
  PosLf2:=PosEx(#10, RequestHeader, PosLf+1);
  while PosLf2>PosLf do
  begin
    sLine:=MidBStr(RequestHeader, PosLf+1, PosLf2-PosLf-1);
    PosColon:=Pos(':', sLine);
    if PosColon>0 then
    begin
      sField:=LeftBStr(sLine, PosColon-1);
      sField:='HTTP_'+UpperCase(StringReplaceB(sField, '-', '_', [rfReplaceAll]));
      with PHP.Variables.Add do
      begin
        Name:=sField;
        Value:=MidBStr(sLine, PosColon+1, Length(sLine));
      end;
    end;
    PosLf:=PosLf2;
    PosLf2:=PosEx(#10, RequestHeader, PosLf+1);
  end;   
end;

procedure TConnectionThread.PHPReadPost(Sender: TObject; Stream: TStream);
begin
  Stream.Write(PChar(RequestPostData)^, Length(RequestPostData));
end;

function GetCodeDescription(Code: Integer): string;
begin
  case Code of
    200: Result:='OK';
    301: Result:='Moved Permanently';   
    302: Result:='Found';
    304: Result:='Not Modified';
    400: Result:='Bad Request';
    401: Result:='Unauthorized';
    403: Result:='Forbidden';
    404: Result:='Not Found';
    405: Result:='Method Not Allowed';
    500: Result:='Internal Server Error';
    501: Result:='Not Implemented';
    503: Result:='Service Unavailable';
  else
    if Code div 100 = 1 then
      Result:='Informational'
    else if Code div 100 = 2 then
      Result:='Success'
    else if Code div 100 = 3 then
      Result:='Moved'
    else if Code div 100 = 4 then
      Result:='Client Error'
    else if Code div 100 = 5 then
      Result:='Server Error'
    else
      Result:='X';
  end;
end;

function IsDenyExt(FileExt, DenyExt: string): Boolean;
begin
  DenyExt:=','+LowerCase(DenyExt)+',';
  if FileExt[1]='.' then
    FileExt:=MidBStr(FileExt, 2, Length(FileExt)-1);
  FileExt:=','+LowerCase(FileExt)+',';
  Result:=Pos(FileExt, DenyExt)>0;
end;

procedure TConnectionThread.PHPReadResult(Sender: TObject; Stream: TStream);
var
  sContentLength: string;
  Headers, sLine, sField: string;
  PosSpace, PosColon, PosLf, PosLf2, Code: Integer;
begin
  Headers:=Trim(StringReplaceB(PHP.Headers.GetHeaders, #13, '', [rfReplaceAll]))+#10;
  if LeftBStr(Headers, 5)<>'HTTP/' then exit;
  if LeftBStr(Headers, 9)='HTTP/1.0 ' then
    Headers[8]:='1';
  // Get response code in first line of Headers, like 'HTTP/1.0 200 X'#10
  PosSpace:=Pos(' ', Headers);
  if PosSpace<=0 then Exit;
  PosLf:=Pos(#10, Headers);
  if (PosSpace+4<PosLf) then
  begin
    try
      Code:=StrToInt(MidBStr(Headers, PosSpace+1, 3));
      sLine:=Format('HTTP/1.1 %0.3d %s'#13#10, [Code, GetCodeDescription(Code)]);
      send(ConnSocket, PChar(sLine)^, Length(sLine), 0);  // Send First Line
    except
      Exit;
    end;
  end else
    Exit;
  PosLf2:=PosEx(#10, Headers, PosLf+1);
  while PosLf2>PosLf do
  begin
    sLine:=MidBStr(Headers, PosLf+1, PosLf2-PosLf-1)+#13#10;
    PosColon:=Pos(':', sLine);
    if PosColon>0 then
    begin
      sField:=LowerCase(LeftBStr(sLine, PosColon-1));
      if (sField<>'server') and (sField<>'x-powered-by') and
        (sField<>'connection') and (sField<>'content-length') then
      begin
        send(ConnSocket, PChar(sLine)^, Length(sLine), 0);  // Send Line
      end;
    end;
    PosLf:=PosLf2;
    PosLf2:=PosEx(#10, Headers, PosLf+1);
  end;
  sLine:='Server: '+GetServerSoftware+#13#10;
  send(ConnSocket, PChar(sLine)^, Length(sLine), 0);  // Send Line
  sLine:='X-Powered-By: Rulzy'#13#10;
  send(ConnSocket, PChar(sLine)^, Length(sLine), 0);  // Send Line
  sLine:='Connection: close'#13#10;
  send(ConnSocket, PChar(sLine)^, Length(sLine), 0);  // Send Line    
  if Stream.Size>0 then
  begin
    sContentLength:=Format('Content-Length: %d'#13#10#13#10, [Stream.Size]);
    send(ConnSocket, PChar(sContentLength)^, Length(sContentLength), 0);
    send(ConnSocket, PChar(TPHPMemoryStream(Stream).Memory)^, Stream.Size, 0);
  end else begin
    send(ConnSocket, PChar(#13#10)^, 2, 0);
  end;
end;

procedure TConnectionThread.ProcessRequest;
var
  FileName, FileExt, Url: string;
  I: Integer;  
  Found: Boolean;
  PVDir: PVirtualDir;
  VDir, Path, DenyExt: string;
begin
  ResponseCode:=200;
  ResponseLocation:='';
  ResponseContentType:='text/html';
  ResponseContent:='';
  if RequestMethod='' then
  begin
    ResponseCode:=405;
    ResponseContent:=GetCodeDescription(405);
    SendResponse;
    Exit;
  end;
  Url:=UrlDecode(Trim(StringReplaceB(RequestUrl, '\', '/', [rfReplaceAll])));
  if Pos('../', Url)>0 then
  begin
    ResponseCode:=302;
    ResponseLocation:='/';
    SendResponse;
    Exit;
  end;
  while RightBStr(Url, 1)='.' do  // remove "." at end of filename
  begin
    SetLength(Url, Length(Url)-1);
    Url:=Trim(Url);
  end;
  if Url='' then
    Url:='/'
  else if Url[1]<>'/' then
    Url:='/'+Url;
  Found:=False;
  for I := 0 to VDirList.Count - 1 do
  begin
    PVDir:=VDirList.Items[I];
    VDir:=PVDir^.VDir;
    Path:=PVDir^.Path;
    DenyExt:=PVDir^.DenyExt;
    if SameText(LeftBStr(Url, Length(VDir)), VDir) then
      FileName:=Path+UTF8Decode(MidBStr(Url, Length(VDir)+1, Length(Url)))
    else if SameText(Url+'/', VDir) then
      FileName:=LeftBStr(Path, Length(Path)-1);
    if DirectoryExists(FileName) then  // is directory
    begin
      Found:=True;
      if RightStr(FileName, 1)='/' then
      begin
        if FileExists(FileName+DEFAULT_SCRIPT_FILE) then
        begin
          FileName:=FileName+DEFAULT_SCRIPT_FILE;
        end else begin
          ResponseCode:=403;
          ResponseContent:=GetCodeDescription(403);
          SendResponse;
          Exit;
        end;
      end else begin
        ResponseCode:=302;
        ResponseLocation:=RequestUrl+'/';
        SendResponse;
        Exit;
      end;
      Break;
    end else if FileExists(FileName) then begin // file exists
      Found:=True;
      Break;
    end;
  end;
  if not Found then
  begin
    ResponseCode:=404;
    ResponseContent:=GetCodeDescription(404);
    SendResponse;
    Exit;
  end;
  FileExt:=AnsiLowerCase(ExtractFileExt(FileName));
  if IsDenyExt(FileExt, DenyExt) then
  begin
    ResponseCode:=403;
    ResponseContent:=GetCodeDescription(403);
    SendResponse;
    Exit;
  end;
  if FileExt=AnsiLowerCase(SCRIPT_FILE_EXT) then
  begin
    try
      if RequestMethod='POST' then
        PHP.RequestType:=prtPost;
      PHP.ContentType:=RequestContentType;
      PHP.QueryString:=RequestGetData;
      PHP.CookieString:=RequestCookies;
      PHP.OnReadPost:=PHPReadPost;
      PHP.OnReadResult:=PHPReadResult;

      RegisterHttpVariables;
      with PHP.Variables.Add do
      begin
        Name:='SCRIPT_FILENAME';
        Value:=FileName;
      end;
      with PHP.Variables.Add do
      begin
        Name:='REMOTE_ADDR';
        Value:=PHP.RequestClientIP;
      end;
      with PHP.Variables.Add do
      begin
        Name:='DOCUMENT_ROOT';
        Value:=g_WebMgrRoot+'www';
      end;
      with PHP.Variables.Add do
      begin
        Name:='SCRIPT_NAME';
        Value:=RequestUrl;
      end;
      with PHP.Variables.Add do
      begin
        Name:='QUERY_STRING';
        Value:=RequestGetData;
      end;
      with PHP.Variables.Add do
      begin
        Name:='REQUEST_URI';
        if RequestGetData='' then
          Value:=RequestUrl
        else
          Value:=RequestUrl+'?'+RequestGetData;
      end;
      with PHP.Variables.Add do
      begin
        Name:='SERVER_NAME';
        Value:='HLDS Web Manager Module';
      end;
      with PHP.Variables.Add do
      begin
        Name:='SERVER_SOFTWARE';
        Value:=GetServerSoftware;
      end;
      with PHP.Variables.Add do
      begin
        Name:='PHP_SELF';
        Value:=RequestUrl;
      end;
      PHP.Execute(FileName);
    except
      ResponseCode:=500;
      ResponseContent:=GetCodeDescription(500);
      SendResponse;
    end;
  end else begin
    if FileExt='.css' then
      ResponseContentType:='text/css'
    else if FileExt='.txt' then
      ResponseContentType:='text/plain'
    else if (FileExt='.htm') or (FileExt='.html') then
      ResponseContentType:='text/html'
    else if (FileExt='.jpg') or (FileExt='.jpe') or (FileExt='.jpeg') then
      ResponseContentType:='image/jpeg'
    else if FileExt='.gif' then
      ResponseContentType:='image/gif'
    else if FileExt='.bmp' then
      ResponseContentType:='image/bmp'
    else if FileExt='.png' then
      ResponseContentType:='image/png'
    else if FileExt='.ico' then
      ResponseContentType:='image/x-icon'
    else
      ResponseContentType:='application/octet-stream';
    try
      SendFile(FileName);
    except
    end;
  end;
end;

procedure TConnectionThread.SendHeaders;
var
  Header: String;
begin
  Header:=Format('HTTP/1.1 %0.3d %s'#13#10, [ResponseCode, GetCodeDescription(ResponseCode)]);
  if ResponseContentType<>'' then
    Header:=Header+'Content-type: '+ResponseContentType+#13#10
  else
    Header:=Header+'Content-type: text/html'#13#10;
  Header:=Header+Format('Server: WebMgr/%s (Win32) PHP/%d.%d.%d'#13#10,
    [MODULE_VERSION, PHPVersion.MajorVersion, PHPVersion.MinorVersion, PHPVersion.Release]);
  Header:=Header+'X-Powered-By: Rulzy'#13#10;
  Header:=Header+'Connection: close'#13#10;
  if ResponseLocation<>'' then
    Header:=Header+'Location: '+ResponseLocation+#13#10;
  send(ConnSocket, PChar(Header)^, Length(Header), 0);
end;

procedure TConnectionThread.SendResponse;
var
  ContentLength: Integer;
  sContentLength: string;
begin
  if FSended then Exit;
  SendHeaders;
  ContentLength:=Length(ResponseContent);
  if ContentLength>0 then
  begin
    sContentLength:=Format('Content-Length: %d'#13#10#13#10, [ContentLength]);
    send(ConnSocket, PChar(sContentLength)^, Length(sContentLength), 0);
    send(ConnSocket, PChar(ResponseContent)^, ContentLength, 0);
  end else begin
    send(ConnSocket, PChar(#13#10)^, 2, 0);
  end;
  FSended:=True;
end;

procedure TConnectionThread.SendFile(FileName: string);
var
  Buf: array[0..8192] of Char;
  FH: THandle;
  ReadBytes, Size: Cardinal;
  sContentLength: string;
begin
  if FSended then Exit;
  SendHeaders;
  FH:=CreateFile(PChar(FileName), GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if FH<>INVALID_HANDLE_VALUE then
  begin
    Size:=GetFileSize(FH, nil);
    if Size<>INVALID_FILE_SIZE then
    begin
      sContentLength:=Format('Content-Length: %d'#13#10#13#10, [Size]);
      send(ConnSocket, PChar(sContentLength)^, Length(sContentLength), 0);
      while True do
      begin
        ReadFile(FH, Buf, 8192, ReadBytes, nil);
        if ReadBytes<8192 then
        begin
          if ReadBytes>0 then
            send(ConnSocket, Buf, ReadBytes, 0);
          Break;
        end;
        send(ConnSocket, Buf, ReadBytes, 0);
      end;
    end else begin 
      send(ConnSocket, PChar(#13#10)^, 2, 0);    
    end;
    CloseHandle(FH);
  end else begin
    send(ConnSocket, PChar(#13#10)^, 2, 0);  
  end;
end;

var
  WSAData:TWSAData;

initialization
  WSAStartup($0101, WSAData);   // Socket ʼ

finalization
  if dmHttpServer<>nil then
    dmHttpServer.Free;
  // ͷ Socket Դ
  WSACleanup;

end.
